<?php
if(!defined('VALID_CMS')) { die('ACCESS DENIED'); }
 
class cms_model_smssend{


    const REQUEST_SUCCESS = 'success';
    const REQUEST_ERROR = 'error';

    protected
        $project    = null,
        $key        = null,
        $testMode   = false,
        $url        = 'mainsms.ru/api/mainsms',
        $useSSL     = false,
        $response   = null;

    /**
     * Конструктор
     *
     * @param string $project
     * @param string $key
     * @param string $useSSL
     * @param integer $testMode
     */
    public function __construct($project, $key, $useSSL = false, $testMode = false)
    {   
	    $this->inDB = cmsDatabase::getInstance();
        $this->project = $project;
        $this->key = $key;
        $this->useSSL = $useSSL;
        $this->testMode = $testMode;
    }

    /**
     * Отправить SMS
     *
     * @param string|array $recipients
     * @param string $message
     * @param string $sender
     *
     * @return boolean|integer
     * @deprecated
     */
    public function sendSMS($recipients, $message, $sender, $run_at = null)
    {
        return $this->messageSend($recipients, $message, $sender, $run_at);
    }

    /**
     * Проверить статус доставки сообщений
     *
     * @param string|array $messagesId
     *
     * @return boolean|array
     * @deprecated
     */
    public function checkStatus($messagesId)
    {
        return $this->messageStatus($messagesId);
    }

    /**
     * Отправить SMS
     *
     * @param string|array $recipients
     * @param string $message
     * @param string $sender
     * @param string $run_at
     *
     * @return boolean|integer
     */
    public function messageSend($recipients, $message, $sender, $run_at = null)
    {
        $params = array(
            'recipients'    => $recipients,
            'message'       => $message,
            'sender'        => $sender,
        );
        
        if ($run_at != null) {
            $params['run_at'] = $run_at;
        }

        if ($this->testMode) {
            $params['test'] = 1;
        }

        $response = $this->makeRequest('message/send', $params);

        return $response['status'] == self::REQUEST_SUCCESS;
    }

    /**
     * Проверить статус доставки сообщений
     *
     * @param string|array $messagesId
     *
     * @return boolean|array
     */
    public function messageStatus($messagesId)
    {
        if (! is_array($messagesId)) {
            $messagesId = array($messagesId);
        }

        $response = $this->makeRequest('message/status', array(
            'messages_id' => join(',', $messagesId),
        ));

        return $response['status'] == self::REQUEST_SUCCESS ? $response['messages'] : false;
    }

    /**
     * Запрос стоимости сообщения
     *
     * @param string|array $recipients
     * @param string $message
     *
     * @return boolean|decimal
     */
    public function messagePrice($recipients, $message)
    {
        $response = $this->makeRequest('message/price', array(
            'recipients'    => $recipients,
            'message'       => $message,
        ));

        return $response['status'] == self::REQUEST_SUCCESS ? $response['price'] : false;
    }

    /**
     * Запрос информации о номерах
     *
     * @param string|array $recipients
     *
     * @return boolean|decimal
     */
    public function phoneInfo($phones)
    {
        $response = $this->makeRequest('message/info', array(
            'phones'    => $phones
        ));

        return $response['status'] == self::REQUEST_SUCCESS ? $response['info'] : false;
    }
    

    /**
     * Запросить баланс
     *
     */
    public function userBalance()
    {
        $response = $this->makeRequest('message/balance');
        return $response['status'] == self::REQUEST_SUCCESS ? $response['balance'] : false;
    }

    /**
     * Запросить баланс
     *
     */
    public function getBalance()
    {
        return $this->userBalance();
    }


    /**
     * Отправить запрос
     *
     * @param string $function
     * @param array $params
     *
     * @return stdClass
     */
    protected function makeRequest($function, array $params = array())
    {
        $params = $this->joinArrayValues($params);
        $sign = $this->generateSign($params);
        $params = array_merge(array('project' => $this->project), $params);

        $url = ($this->useSSL ? 'https://' : 'http://') . $this->url .'/'. $function;
        $post = http_build_query(array_merge($params, array('sign' => $sign)), '', '&');
		
        if (function_exists('curl_init')) {
            $ch = curl_init($url);
            if ($this->useSSL) {
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
                curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            }
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

            $response = curl_exec($ch);
            curl_close($ch);
        } else {
            $context = stream_context_create(array(
                'http' => array(
                    'method' => 'POST',
                    'header' => "Content-type: application/x-www-form-urlencoded\r\n",
                    'content' => $post,
                    'timeout' => 10,
                ),
            ));
            $response = file_get_contents($url, false, $context);
        }
        return $this->response = json_decode($response, true);
    }


    /**
     * Установить адрес шлюза
     *
     * @param string $url
     * @return void
     */
    public function setUrl($url)
    {
        $this->url = $url;
    }


    /**
     * Получить адрес сервера
     *
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    protected function joinArrayValues($params)
    {
        $result = array();
        foreach ($params as $name => $value) {
            $result[$name] = is_array($value) ? join(',', $value) : $value;
        }
        return $result;
    }


    /**
     * Возвращает ответ сервера последнего запроса
     *
     * @return array
     */
    public function getResponse()
    {
        return $this->response;
    }


    /**
     * Сгенерировать подпись
     *
     * @param array $params
     * @return string
     */
    protected function generateSign(array $params)
    {
	    $params['project'] = $this->project;
	    ksort($params);
	    return md5(sha1(join(';', array_merge($params, Array($this->key)))));
    }
	
	
	/* 
	* Заносим сообщения в базу данных
	*/	
	public function addSmstext($user_id, $sms_id, $sender, $recipients, $text, $true_price, $parts, $price) {
 
        //готовим запрос для добавления записи
        $sql = "INSERT INTO cms_smssend (user_id, sms_id, sender, recipients, text, true_price, parts, price, pubdate )
                VALUES ('{$user_id}', '{$sms_id}', '{$sender}', '{$recipients}', '{$text}', '{$true_price}', '{$parts}', '{$price}', NOW())";
 
        //выполняем запрос
        $this->inDB->query($sql);
 
        //если возникла ошибка, вернем false
        if ($this->inDB->error()) { return false; }
 
        //если ошибок не было, вернем ID новой записи
        return $this->inDB->get_last_id('cms_smssend');
 
    }
	
	/* 
	* Обновляем баланс
	*/	
	public function updateBalance($user_id, $price) {
 
        $sql = "UPDATE cms_users SET balance = balance - '{$price}' WHERE id = '{$user_id}' ";

        $this->inDB->query($sql);
 
    }
	
	/* 
	* Заносим в базу купленные пакеты
	*/	
	public function addSmsPackage($user_id, $amount, $cost) {
 
        //готовим запрос для добавления записи
        $sql = "INSERT INTO cms_smssend_package (user_id, amount, cost, pubdate )
                VALUES ('{$user_id}', '{$amount}', '{$cost}', NOW())";
 
        //выполняем запрос
        $this->inDB->query($sql);
 
        //если возникла ошибка, вернем false
        if ($this->inDB->error()) { return false; }
 
        //если ошибок не было, вернем ID новой записи
        return $this->inDB->get_last_id('cms_smssend_package');
 
    }
	
	//Обновляем баланс смсок
	
	public function updateSmsBalance($user_id, $amount) {
 
        $sql = "UPDATE cms_users SET sms_balance = sms_balance + '{$amount}' WHERE id = '{$user_id}' ";

        $this->inDB->query($sql);
 
    }
	
	//Обновляем баланс биллинга после покупки пакетов смсок
	public function updateBal_sms($user_id, $cost) {
 
        $sql = "UPDATE cms_users SET balance = balance - '{$cost}' WHERE id = '{$user_id}' ";

        $this->inDB->query($sql);
 
    }
	
	//обновляем баланс смсок, после отправлений
	public function updateSB($user_id, $parts) {
 
        $sql = "UPDATE cms_users SET sms_balance = sms_balance - '{$parts}' WHERE id = '{$user_id}' ";

        $this->inDB->query($sql);
 
    }
	
	//Добавляем в базу статус смски (если пользователь запросил статус)
	
	public function addStatus($sms_id, $st_mes) {
 
        $sql = "UPDATE cms_smssend SET status = '{$st_mes}' WHERE sms_id = '{$sms_id}'	";
 
        $this->inDB->query($sql);
    }
	
	
	
	/* 
	* Выбираем сообщения из базы, для показа в истории
	*/	
	public function getSms() {
 
        $inCore = cmsCore::getInstance();
 
        $sql = "SELECT *
                FROM `cms_smssend`
				ORDER BY id DESC
				";
 
        $result = $this->inDB->query($sql);
 
        //if ($this->inDB->error()) { return false; } //опционально
 
        //if (!$this->inDB->num_rows($result)) { return false; }  // лишния часть, т.к в контроллере можно проверить не пустой ли результат функции
 
        $sms_mess = array();
        $i = 0;
        while ($item = $this->inDB->fetch_assoc($result)){
 
            $sms_mess[$i] = $item;
            $sms_mess[$i]['pubdate'] = $inCore->dateFormat($item['pubdate']);
            $i++;
        }
 
        return $sms_mess;
 
	}
	
	// Берем число оставшихся смс на балансе
	public function getSmsBalance() {
 
        $inCore = cmsCore::getInstance();
 
        $sql = "SELECT id, sms_balance
                FROM `cms_users`";
				
        $result = $this->inDB->query($sql);
 
        $balsms = array();
		$i = 0;
        while ($item = $this->inDB->fetch_assoc($result)){
 
            $balsms[$i] = $item;
            $i++;
        }

 
        return $balsms;
 
	}
	
	
}

?>






